//----------------------------------------------------------------------------
//  Project WmPlayer
//  OWLNext community
//  (C) 2007 OWLNext community
//
//  SUBSYSTEM:    WmPlayer Application
//  FILE:         wmplyrdc.cpp
//  AUTHOR:       Sebastian Ledesma
//
//  OVERVIEW
//  ~~~~~~~~
//  Source file for implementation of TWmPlayerDlgClient (TDialog).
//
//----------------------------------------------------------------------------

#include <owl/pch.h>
#include <owl/opensave.h>
#include <owl/profile.h>

#include "wmplyrap.h"
#include "wmplyrdc.h"


//
// Build a response table for all messages/commands handled by the application.
//
DEFINE_RESPONSE_TABLE1(TWmPlayerDlgClient, TOleDialog)
//{{TWmPlayerDlgClientRSP_TBL_BEGIN}}
  EV_WM_SIZE,
  EV_BN_CLICKED(IDC_OPEN, OnOpen),
  EV_BN_CLICKED(IDC_PLAY, OnPlay),
  EV_BN_CLICKED(IDC_STOP, OnStop),
  EV_WM_PAINT,
  EV_WM_LBUTTONDOWN,
  EV_WM_MOUSEMOVE,
  EV_WM_LBUTTONUP,
  EV_BN_CLICKED(IDC_VOLUME_UP, OnVolumeUp),
  EV_BN_CLICKED(IDC_VOLUME_DOWN, OnVolumeDown),

  EV_BN_CLICKED(IDC_UP, OnUp),
  EV_BN_CLICKED(IDC_DOWN, OnDown),
  EV_BN_CLICKED(IDC_LEFT, OnLeft),
  EV_BN_CLICKED(IDC_RIGHT, OnRight),
//{{TWmPlayerDlgClientRSP_TBL_END}}
END_RESPONSE_TABLE;


//{{TWmPlayerDlgClient Implementation}}


//--------------------------------------------------------
// TWmPlayerDlgClient
// ~~~~~~~~~~
// Construction/Destruction handling.
//
//static TWmPlayerDlgClientXfer TWmPlayerDlgClientData;

TWmPlayerDlgClient::TWmPlayerDlgClient(TWindow* parent, TResId resId, TModule* module)
:
  TOleDialog(parent, resId, module) {
//{{TWmPlayerDlgClientXFER_USE}}
  pBtn_Open = new TButton(this, IDC_OPEN);
  pBtn_Play = new TButton(this, IDC_PLAY);
  pBtn_Stop = new TButton(this, IDC_STOP);
  pBtn_Ok = new TButton(this, IDOK);
  pBtn_Volumeup = new TButton(this, IDC_VOLUME_DOWN);
  pBtn_VolumeDown = new TButton(this, IDC_VOLUME_UP);

//  SetTransferBuffer(&TWmPlayerDlgClientData);
//{{TWmPlayerDlgClientXFER_USE_END}}

  // INSERT>> Your constructor code here.

  // Create TOcDocument object to hold OLE parts
  // and TOcView object to provide OLE services.
  // CreateOcView(0, false, 0);
  //Not needed, will be called in TOleWindow::Setup if not called here

	//SetBkgndColor(RGB(0x80, 0xA0, 0xe0)); //8f af cf
   Ocx=NULL;

	Sliding=false;
	hRegionComplete=NULL;
	hDrawRegion=NULL;
   fondo=NULL;
}


TWmPlayerDlgClient::~TWmPlayerDlgClient() {
//  Destroy(); not needed, it will called by TOleDialog destructor

  // INSERT>> Your destructor code here.
}

void TWmPlayerDlgClient::SetupWindow() {
	TOleDialog::SetupWindow();

	// Note 1: the ProgID must be used as caption in the control
	// "MediaPlayer.MediaPlayer.1" for classic Media Player (msdxm.ocx)
	// "WMPlayer.OCX.7" for new Media Player (wmp.dll)
	GUID  guidMediaPlayer = CLSID_NULL;

#define USE_PROGID_TO_OBTAIN_GUID 1
#if USE_PROGID_TO_OBTAIN_GUID
	CLSIDFromProgID(TString("MediaPlayer.MediaPlayer.1"), &guidMediaPlayer); //classic Media player
	if (guidMediaPlayer==CLSID_NULL)
		MessageBox("MediaPlayer not installed in this system","Error",MB_OK);
#else
	guidMediaPlayer=IID_IMediaPlayer2; //classic Media player GUID
   // guidMediaPlayer= {6BF52A52-394A-11D3-B153-00C04F79FAA6} //Media Player 7-9 GUID
#endif
	Ocx = GetOcControlOfOCX(guidMediaPlayer, IDC_MEDIAPLAYER);
	if (!Ocx) {
//		MessageBox("Can't find to OCX control using GUID and control ID, switch to GUID only mode", "Update to OWLNext 6.20.10 or above", MB_OK);
		Ocx = GetOcControlOfOCX(guidMediaPlayer);
	}

	if (Ocx)
		Proxy.Bind(Ocx->GetCtlDispatch());
	else
		MessageBox("Can't bind to OCX control.\r\nMedia Player not correctly installed?", "Error", MB_OK);


   //Resize the main window (TOleFrame) to the dialog client
	/*TRect drect;
	TRect srect;
	GetWindowRect(drect);
	AdjustWindowRectEx(drect, Parent->GetStyle(), false, Parent->GetExStyle());
	Parent->SetWindowPos(0, drect, SWP_NOMOVE|SWP_NOZORDER|SWP_SHOWWINDOW);      */

	loadSkin(""); //no subdir used
	if (Ocx) { //We cannot use SetWindowPos on a OcControl
      Proxy.ShowControls(false);
		Proxy.EnablePositionControls(true);//Enables or disables the position buttons in the control panel  [dispid 27]
//		Proxy.EnableTracker(true);	// Enables or disables the tracker bar in the control panel  [dispid 29]

//		Proxy.DisplayBackColor(VB_OLE_COLOR arg0);
//		Proxy.DisplayBackColor(0);

	// VideoBorderColor: Returns or sets the video border color  [dispid 1102]
//		Proxy.VideoBorderColor(VB_OLE_COLOR arg0);
	}

}


void TWmPlayerDlgClient::CleanupWindow() {
  // INSERT>> Your code here.
  TOleDialog::CleanupWindow();
  if (hDrawRegion) {
		DeleteObject(hDrawRegion);
   	hDrawRegion=NULL;
  }
}


void TWmPlayerDlgClient::EvSize(uint sizeType, TSize& size) {
	TOleDialog::EvSize(sizeType, size);

	// INSERT>> Your code here.
	/*if (sizeType != SIZEICONIC ) {
	  	if (Ocx) {
  			//Ocx->SetWindowPos(0, 10, 10, size.cx-10, size.cy-40-10, SWP_NOZORDER | SWP_NOMOVE);
      	Ocx->SetWidth(size.cx-10);
	      Ocx->SetHeight(size.cy-40-10);
      }

		pBtn_Open->SetWindowPos(0, 10, size.cy-30, 0, 0, SWP_NOZORDER | SWP_NOSIZE);
		pBtn_Play->SetWindowPos(0, 100, size.cy-30, 0, 0, SWP_NOZORDER | SWP_NOSIZE);
		pBtn_Stop->SetWindowPos(0, 190, size.cy-30, 0, 0, SWP_NOZORDER | SWP_NOSIZE);
		pBtn_Ok->SetWindowPos(0, 280, size.cy-30, 0, 0, SWP_NOZORDER | SWP_NOSIZE);
	}   */
}

void TWmPlayerDlgClient::OnOpen(){
  // INSERT>> Your code here.
  char filename[MAX_PATH];
  TFileOpenDialog::TData FilenameData (OFN_FILEMUSTEXIST | OFN_PATHMUSTEXIST | OFN_NOCHANGEDIR,
	    "Media files (*.avi;*.mp?;*.wav)|*.avi;*.mp?;*.wav|AVI Files (*.avi)|*.avi|Wave Files (*.wav)|*.wav|MPEG Files (*.mp?)|*.mp?|",
   	  0, filename, 0, 0 /*4096*/);
  if (TFileOpenDialog(this, FilenameData).Execute() != IDOK)
			return;

  //TBSTR bstrFilename;     //defined in <ocf/oleutil.h>
  //TSysStr sysStrFilename; //defined in <winsys/string.h>

  //BSTR  bstrFilename=::SysAllocString(L"DEMO.MP3");

  wchar_t wFilename[MAX_PATH];
  MultiByteToWideChar(CP_ACP, MB_PRECOMPOSED, FilenameData.FileName, -1, wFilename, MAX_PATH);

  //TR  bstrFilename=::SysAllocString(FilenameData.FileName);
  BSTR  bstrFilename=::SysAllocString(wFilename);

  Proxy.Open(bstrFilename);

  ::SysFreeString(bstrFilename);
}


void TWmPlayerDlgClient::OnPlay() {
  // INSERT>> Your code here.
  Proxy.Play();
}


void TWmPlayerDlgClient::OnStop() {
  // INSERT>> Your code here.
  Proxy.Stop();
}


bool TWmPlayerDlgClient::CanClose() {
	if (Ocx) {
   	Proxy.Stop();
		Proxy.Unbind();
      Ocx=NULL;
   }

  bool result;
  result = TOleDialog::CanClose();
  // INSERT>> Your code here.

  return result;
}


int TWmPlayerDlgClient::loadSkin(char *skinName) {
  // INSERT>> Your code here.
	int newSkin=false;
	char skinDir[MAX_PATH], auxFName[MAX_PATH];
   char iniFile[MAX_PATH];
	strcpy(skinDir,".\\");

	strcpy(auxFName, skinDir);
	strcat(auxFName, skinName);
	strcat(auxFName, "\\");
   strcpy(iniFile, auxFName);
	strcat(iniFile, "OWLPlayer.ini");
	strcat(auxFName, "body.bmp");
	if (fondo) {
		delete fondo;
		fondo=NULL;
	}
	try {
		fondo = new TDib(auxFName);
		newSkin=1;
	}
	catch(TXOwl& exp){
		fondo =NULL;
		//use resource bitmap
#if OWLVersion <0x0600
		fondo = new TDib(GetApplication()->GetInstance(), TResId(IDB_FONDO));
#else
		//For OWLNext you have two options:
		//option 1 explicit module handle:
		fondo = new TDib(GetApplication()->GetHandle(), TResId(IDB_FONDO));
		//optiom 2 implicit module handle
		//fondo = new TDib(NULL, TResId(IDB_FONDO));
#endif
	}

   TProfile ini(iniFile,"Settings");

	xDisplay=ini.GetInt("xDisplay",162);
	yDisplay=ini.GetInt("yDisplay",46);
   wDisplay=ini.GetInt("wDisplay",522);
   hDisplay=ini.GetInt("hDisplay",284);
   allowHolesInSkin=ini.GetInt("allowHolesInSkin",0);
	maskColor=ini.GetInt("maskColor",192);

   xPlay=ini.GetInt("xPlay",802);
   yPlay=ini.GetInt("yPlay",175);
   xStop=ini.GetInt("xStop",730);
   yStop=ini.GetInt("yStop",175);
   xClose=ini.GetInt("xClose",768);
   yClose=ini.GetInt("yClose",136);
   xOpen=ini.GetInt("xOpen",768);
   yOpen=ini.GetInt("yOpen",212);

   xUp=ini.GetInt("xUp",76);
   yUp=ini.GetInt("yUp",136);
   xLeft=ini.GetInt("xLeft",36);
   yLeft=ini.GetInt("yLeft",175);
   xRight=ini.GetInt("xRight",117);
   yRight=ini.GetInt("yRight",175);
   xDown=ini.GetInt("xDown",76);
   yDown=ini.GetInt("yDown",212);

   xVolumeUp=ini.GetInt("xVolumeUp",460);
   yVolumeUp=ini.GetInt("yVolumeUp",361);
   xVolumeDown=ini.GetInt("xDown",380);
   yVolumeDown=ini.GetInt("yDown",361);

   buttonRadio=32;

	if (Ocx) { //We cannot use SetWindowPos on a OcControl
      Ocx->SetTop(yDisplay);	//use specific API
      Ocx->SetLeft(xDisplay);
		Ocx->SetWidth(wDisplay);
	   Ocx->SetHeight(hDisplay);
	}

	if (fondo && !fondo->Compression()) {
		pBtn_Open->ShowWindow(SW_HIDE);
		pBtn_Play->ShowWindow(SW_HIDE);
		pBtn_Stop->ShowWindow(SW_HIDE);
		pBtn_Ok->ShowWindow(SW_HIDE);
		pBtn_Volumeup->ShowWindow(SW_HIDE);
      pBtn_VolumeDown->ShowWindow(SW_HIDE);

		int r, g, b;
		BYTE *pixel;
		int paletteIndex;
		TColor auxColor(0,0,0);

		int regionVacia=1;
		hDrawRegion=CreateRectRgn( 0, 0, 0, 0);
		hRegionComplete=CreateRectRgn( 0, 0, fondo->Width(), fondo->Height());
		HRGN hRegion;
		int x, y, y2;
		for (x=0; x<fondo->Width(); x++) {
	      for (y=0; y<fondo->Height(); y++) {
   	     pixel=(BYTE *)fondo->PixelPtr(x, fondo->FlippedY(y));
      	  switch (fondo->BitsPixel()) {
         	 case 32:
	          case 24:
   	       b= *pixel;
      	    g= *(pixel+1);
         	 r= *(pixel+2);
	          break;
   	     case 16:
      	    b= *pixel & 0x1F; //me quedo con los 5 bits mas bajos
         	 g= ((*pixel & 0xE0) >>5) | ((*(pixel+1) & 0x03) <<5) ; //los 3 bits mas altos del anterior y los 2 mas bajos del siguiente
	          r= (*(pixel+1) & 0x7C) >> 2; //5 bits del medio
   	       break;
      	  case 8:
	          paletteIndex=*pixel;
   	       auxColor=fondo->GetColor(paletteIndex);
      	    r=auxColor.Red();
	          g=auxColor.Green();
   	       b=auxColor.Blue();
      	    break;
	        case 4:
   	       if (y%2)
      	      paletteIndex=*pixel & 0x0f;
	          else
   	         paletteIndex=*pixel & 0xf0;
	          auxColor=fondo->GetColor(paletteIndex);
   	       r=auxColor.Red();
      	    g=auxColor.Green();
	          b=auxColor.Blue();
   	       break;
      	  case 2:
	          //no soportado
   	       int i= 2 ^ x %8; //i=1 o 2 o 4 o 8 o 16 o 32 o 64 o 128
	          if (*pixel & i) { //esta prendido el bit?
   	         r=g=b=255;
	          }
   	       else {
      	      r=g=b=0;
         	 }
	          break;
			}

			if (r==g && g==b && b==maskColor) {
          	hRegion=CreateRectRgn( x, y, x+1, y+1);
          	CombineRgn(hRegionComplete, hRegionComplete, hRegion, RGN_XOR);
            DeleteObject(hRegion);
            regionVacia=0;
			}
			else if (!allowHolesInSkin) {
        		break;
			}
      }
		if (allowHolesInSkin==false && y<fondo->Height()) {
			for (y2=fondo->Height()-1; y2>y; y2--) {
				pixel=(BYTE *)fondo->PixelPtr(x, fondo->FlippedY(y2));
				switch (fondo->BitsPixel()) {
	          case 32:
   	       case 24:
      	    b= *pixel;
	          g= *(pixel+1);
   	       r= *(pixel+2);
      	    break;
	        case 16:
   	       b= *pixel & 0x1F; //me quedo con los 5 bits mas bajos
	          g= ((*pixel & 0xE0) >>5) | ((*(pixel+1) & 0x03) <<5) ; //los 3 bits mas altos del anterior y los 2 mas bajos del siguiente
   	       r= (*(pixel+1) & 0x7C) >> 2; //5 bits del medio
      	    break;
	        case 8:
   	       paletteIndex=*pixel;
      	    auxColor=fondo->GetColor(paletteIndex);
	          r=auxColor.Red();
   	       g=auxColor.Green();
      	    b=auxColor.Blue();
	          break;
	        case 4:
   	       if (y%2)
	            paletteIndex=*pixel & 0x0f;
   	       else
	            paletteIndex=*pixel & 0xf0;
   	       auxColor=fondo->GetColor(paletteIndex);
	          r=auxColor.Red();
   	       g=auxColor.Green();
      	    b=auxColor.Blue();
	          break;
   	     case 2:
	          //no soportado
   	       int i= 2 ^ x %8; //i=1 o 2 o 4 o 8 o 16 o 32 o 64 o 128
      	    if (*pixel & i) { //esta prendido el bit?
	            r=g=b=255;
   	       }
      	    else {
	            r=g=b=0;
   	       }
	          break;
         }
         if (r==g && g==b && b==maskColor) {
				hRegion=CreateRectRgn( x, y2, x+1, y2+1);
          	CombineRgn(hRegionComplete, hRegionComplete, hRegion, RGN_XOR);
            DeleteObject(hRegion);
            regionVacia=0;
			}
			else
        		break;
	      }
     	}
		}

		if (hDrawRegion) //creo una copia para la rutina de dibujo en el area NC
			CombineRgn(hDrawRegion, hDrawRegion, hRegionComplete, RGN_OR);

   	if( !regionVacia) {
	    	SetWindowRgn(GetApplication()->GetMainWindow()->GetHandle(), hRegionComplete,true);
		}
//		GetApplication()->GetMainWindow()->SetWindowPos( 0, 0, 0, fondo->Width(), fondo->Height(), SWP_NOMOVE);
		int borderSize=4;
		SetWindowPos( 0, 0, 0, fondo->Width()+borderSize, fondo->Height()+borderSize, SWP_NOMOVE);
	}
	else {
		pBtn_Open->ShowWindow(SW_SHOW);
		pBtn_Play->ShowWindow(SW_SHOW);
		pBtn_Stop->ShowWindow(SW_SHOW);
		pBtn_Ok->ShowWindow(SW_SHOW);

		pBtn_Volumeup->ShowWindow(SW_SHOW);
      pBtn_VolumeDown->ShowWindow(SW_SHOW);
  }
return newSkin;
}


void TWmPlayerDlgClient::EvPaint() {
//    TOleDialog::EvPaint();
  // INSERT>> Your code here.
	TRect r=GetClientRect();
	TPaintDC  dc(*this);
	Paint(dc, false, r);
}

void TWmPlayerDlgClient::Paint(TDC& dc, bool /*erase*/, TRect& rect) {
  // INSERT>> Your code here.
/*#if OWLVersion <0x0600
	TPoint start(0,0);
#else
	int borderSize=4;
	TPoint start(0,0);
#endif*/
	TPoint start(0,0);
	if (fondo && fondo->Height() && fondo->Width()) {
		dc.SetDIBitsToDevice(rect, start, *fondo);
	}
//	TOleDialog::Paint(dc, erase, rect);
}


void TWmPlayerDlgClient::EvLButtonDown(uint /*modKeys*/, TPoint& point) {
//    TOleDialog::EvLButtonDown(modKeys, point);
  // INSERT>> Your code here.
	if(fondo && checkVirtualButton(point))
   	return;

	SetCapture();
	Sliding=true;
	referencia.x=point.x;
	referencia.y=point.y;
}


void TWmPlayerDlgClient::EvMouseMove(uint /*modKeys*/, TPoint& point) {
//    TOleDialog::EvMouseMove(modKeys, point);
  // INSERT>> Your code here.
	if (Sliding) {
		ClientToScreen(point); //traduce la coordenada local a global
		TRect r = GetWindowRect();
		r.Set(point.x - referencia.x, point.y - referencia.y, 0, 0);
		r.bottom=0;
		r.right=0;
		GetApplication()->GetMainWindow()->SetWindowPos(0, r, SWP_NOSIZE|SWP_NOZORDER);
  }
}


void TWmPlayerDlgClient::EvLButtonUp(uint /*modKeys*/, TPoint& point) {
//    TOleDialog::EvLButtonUp(modKeys, point);
	// INSERT>> Your code here.
	if(Sliding){
		Sliding = false;
		ReleaseCapture();
      return;
	}
   if (fondo) {
	   int cmd=checkVirtualButton(point);
		if(cmd)
	  		PostMessage(WM_COMMAND,cmd,0);
   }
}

int distance(int x1, int y1, int x2, int y2) {
	return (x1-x2)*(x1-x2) + (y1-y2)*(y1-y2);
}

int TWmPlayerDlgClient::checkVirtualButton(TPoint &p){
int hipotenusaCuadrada=buttonRadio*buttonRadio;
	if (distance(p.x, p.y, xPlay, yPlay)<hipotenusaCuadrada)
   	return IDC_PLAY;
	if (distance(p.x, p.y, xStop, yStop)<hipotenusaCuadrada)
   	return IDC_STOP;
	if (distance(p.x, p.y, xOpen, yOpen)<hipotenusaCuadrada)
   	return IDC_OPEN;
	if (distance(p.x, p.y, xClose, yClose)<hipotenusaCuadrada)
   	return IDOK;

	if (distance(p.x, p.y, xVolumeUp, yVolumeUp)<hipotenusaCuadrada)
   	return IDC_VOLUME_UP;
	if (distance(p.x, p.y, xVolumeDown, yVolumeDown)<hipotenusaCuadrada)
   	return IDC_VOLUME_DOWN;

	if (distance(p.x, p.y, xUp, yUp)<hipotenusaCuadrada)
   	return IDC_UP;
	if (distance(p.x, p.y, xDown, yDown)<hipotenusaCuadrada)
   	return IDC_DOWN;
	if (distance(p.x, p.y, xLeft, yLeft)<hipotenusaCuadrada)
   	return IDC_LEFT;
	if (distance(p.x, p.y, xRight, yRight)<hipotenusaCuadrada)
   	return IDC_RIGHT;

	return 0;
}

void TWmPlayerDlgClient::OnVolumeUp()
{
  // INSERT>> Your code here.
   if (Ocx) {
		long volume=Proxy.Volume();
      volume=volume+500;
      if (volume>0)
      	volume=0;
		Proxy.Volume(volume);
   }
}


void TWmPlayerDlgClient::OnVolumeDown()
{
  // INSERT>> Your code here.
   if (Ocx) {
		long volume=Proxy.Volume();
      volume=volume-500;
      if (volume<-32768)
      	volume=32768;
		Proxy.Volume(volume);
   }
}


void TWmPlayerDlgClient::OnUp(){
   ShellExecute(NULL, "open", "http://owlnext.sourceforge.net", "", "", SW_SHOWNORMAL );
}

void TWmPlayerDlgClient::OnDown(){
   ShellExecute(NULL, "open", "http://www.solidynePro.com", "", "", SW_SHOWNORMAL );
}

void TWmPlayerDlgClient::OnLeft(){
   ShellExecute(NULL, "open", "http://www.google.com", "", "", SW_SHOWNORMAL );
}

void TWmPlayerDlgClient::OnRight(){
   ShellExecute(NULL, "open", "http://www.codegear.com", "", "", SW_SHOWNORMAL );
}
